var toCamelCase = require('to-camel-case')

var gm = require('./gm')

var slice = [].slice

/*

  Convert the list from the site to an object

  function listToObj(str) {
    var obj = {}

    str.replace(/ • /g, ' ').split(/\s+/).forEach(function (x) {
      obj[x.slice(1)] = true
    })

    return obj
  }

*/

// http://www.imagemagick.org/script/command-line-processing.php#setting
var settings = {
  adjoin: true,
  affine: true,
  alpha: true,
  antialias: true,
  authenticate: true,
  background: true,
  bias: true,
  'black‑point‑compensation': true,
  'blue‑primary': {
    join: ','
  },
  bordercolor: true,
  caption: true,
  channel: true,
  comment: true,
  compress: true,
  debug: true,
  define: true,
  delay: true,
  density: true,
  depth: true,
  direction: true,
  displace: { // http://www.graphicsmagick.org/GraphicsMagick.html#details-displace
    join: ','
  },
  display: true,
  dispose: true,
  dither: true,
  encoding: true,
  endian: true,
  extract: true,
  family: true,
  fill: true,
  filter: true,
  font: true,
  format: true,
  fuzz: true,
  geometry: true,
  gravity: true,
  'green‑primary': true,
  interlace: true,
  intent: true,
  interpolate: true,
  label: true,
  limit: true,
  linewidth: true,
  log: true,
  loop: true,
  mask: true,
  mattecolor: true,
  monitor: true,
  orient: true,
  page: true,
  pointsize: true,
  preview: true,
  quality: true,
  quiet: true,
  'red‑primary': true,
  region: true,
  render: true,
  repage: true,
  'sampling‑factor': true,
  scene: true,
  seed: true,
  size: true,
  stretch: true,
  stroke: true,
  strokewidth: true,
  style: true,
  texture: true,
  tile: true,
  'transparent‑color': true,
  treedepth: true,
  type: true,
  undercolor: true,
  units: true,
  verbose: true,
  'virtual‑pixel': true,
  weight: true,
}

var operators = {
  // http://www.imagemagick.org/script/command-line-processing.php#operator
  annotate: true,
  'black‑threshold': {
    join: ','
  },
  blur: true,
  border: {
    join: 'x'
  },
  charcoal: true,
  chop: true,
  clip: true,
  'clip‑path': true,
  'clip‑mask': true,
  colors: true,
  colorize: {
    join: ','
  },
  colorspace: true,
  compose: true,
  contrast: true,
  convolve: true,
  crop: true,
  cycle: true,
  despeckle: true,
  draw: true,
  edge: true,
  emboss: true,
  enhance: true,
  equalize: true,
  evaluate: true,
  extent: true,
  flip: true,
  flop: true,
  floodfill: true,
  frame: true,
  gamma: true,
  'gaussian‑blur': true,
  implode: true,
  lat: true,
  level: true,
  map: true,
  median: true,
  modulate: true,
  monochrome: true,
  negate: true,
  noise: true,
  normalize: true,
  opaque: true,
  'ordered‑dither': true,
  paint: true,
  posterize: true,
  raise: true,
  profile: true,
  'radial‑blur': true,
  'random‑threshold': true,
  resample: true,
  resize: true,
  roll: true,
  rotate: true,
  sample: true,
  scale: true,
  'sepia‑tone': true,
  segment: true,
  shade: true,
  shadow: true,
  sharpen: true,
  shave: true,
  shear: true,
  'sigmoidal‑contrast': true,
  solarize: true,
  splice: true,
  spread: true,
  strip: true,
  swirl: true,
  threshold: true,
  transparent: true,
  thumbnail: true,
  tint: true,
  transform: true,
  trim: true,
  unsharp: true,
  version: true,
  wave: true,
  'white‑point': true,
  'white‑threshold': true,
  // http://www.imagemagick.org/script/command-line-processing.php#sequence
  append: true,
  affinity: true,
  average: true,
  clut: true,
  coalesce: true,
  combine: true,
  compare: true,
  complex: true,
  composite: true,
  crop: true,
  debug: true,
  deconstruct: true,
  delete: true,
  'evaluate‑sequence': true,
  fft: true,
  flatten: true,
  fx: true,
  'hald‑clut': true,
  ift: true,
  identify: true,
  insert: true,
  layers: true,
  limit: true,
  map: true,
  maximum: true,
  minimum: true,
  morph: true,
  mosaic: true,
  optimize: true,
  print: true,
  process: true,
  quiet: true,
  separate: true,
  swap: true,
  write: true,
  // http://www.imagemagick.org/script/command-line-processing.php#geometry
  'adaptive‑resize': true,
  border: true,
  borderwidth: true,
  chop: true,
  crop: true,
  density: true,
  extent: true,
  extract: true,
  frame: true,
  geometry: true,
  iconGeometry: true,
  'liquid‑rescale': true,
  page: true,
  region: true,
  repage: true,
  resize: true,
  sample: true,
  scale: true,
  shave: true,
  splice: true,
  thumbnail: true,
  window: true,
}

Object.keys(settings).forEach(function (setting) {
  var options = settings[setting]

  gm.prototype[toCamelCase(setting)] = function (arg) {
    this.setting(setting)
    if (typeof arg === 'number' && options.join)
      this.setting(join(arguments, options.join))
    else
      this.setting.apply(this.setting, arguments)
    return this
  }
})

Object.keys(operators).forEach(function (operator) {
  var options = operators[operator]

  gm.prototype[toCamelCase(operator)] = function (arg) {
    this.operator(operator)
    if (typeof arg === 'number' && options.join)
      this.operator(join(arguments, options.join))
    else
      this.operator.apply(this.operator, arguments)
    return this
  }
})

// Custom arguments
gm.prototype.chop = function () {
  return this.setting('-chop', geometry(arguments))
}

gm.prototype.frame = function () {
  if (!arguments.length)
    return this.operator('-frame')
  return this.operator('-frame', geometry(arguments))
}

// For joining numbers
function join(args, delim) {
  return slice.call(args, 0).join(delim)
}

function geometry(args) {
  return args[0] + 'x'
    + args[1] + '+'
    + (args[2] || 0) + '+'
    + (args[3] || 0)
}