var identifyParser = require('imagemagick-identify-parser')
var debug = require('debug')('gm')

var gm = require('./gm')

// http://www.imagemagick.org/www/escape.html
// Feel free to add more!
// Only returns the data of the first frame.
var formatters = {
  getFormat: {
    format: '%m'
  },
  getFilesize: {
    format: '%m',
    formatter: toInteger
  },
  getQuantumDepth: {
    format: '%q',
    formatter: toInteger
  },
  getSize: {
    format: '%G',
    formatter: function (str) {
      var frags = str.split('x')
      return {
        width: toInteger(frags.shift()),
        height: toInteger(frags.shift())
      }
    }
  },
  getColors: {
    format: '%k',
    formatter: toInteger
  },
  getOrientation: {
    format: '%[orientation]',
    formatter: toInteger
  },
  getSignature: {
    format: '%#'
  }
}

gm.prototype.identify = function (format, options, done) {
  if (typeof format === 'function') {
    done = format
    format = null
    options = {}
  } else if (typeof format === 'object') {
    options = format
    format = null
  }

  // Backwards compatibility with buffer stream
  if (options.bufferStream)
    this.buffer(options.bufferStream)

  var command = 'identify'
  var args = format
    ? ['-format', format, this.source]
    : ['-verbose', this.source]

  debug('identify: ' + args.join(' '))

  this.spawn(command, args, options)

  this.toBuffer(function (err, buf) {
    if (err)
      done.call(this, err)
    else if (format)
      done.call(this, null, buf)
    else
      done.call(this, null, identifyParser(buf.toString('utf8'), true))
  })

  return function (fn) {
    done = fn
  }
}

Object.keys(formatters).forEach(function (key) {
  var options = formatters[key]
  var format = options.format + '|'
  var formatter = options.formatter

  gm.prototype[key] = function (options, done) {
    if (typeof options === 'function') {
      done = options
      options = {}
    }

    this.identify(format, options, function (err, buf) {
      if (err)
        return done.call(this, err)

      buf = buf.toString('utf8').split('|').shift()
      if (formatter)
        buf = formatter(buf)

      done.call(this, null, buf)
    })

    return function (fn) {
      done = fn
    }
  }
})

function toInteger(str) {
  return isNaN(str) ? null : parseInt(str, 10)
}